require "stringio"
class SVG
  def plot 
    return Plot.new
  end 

  class Plot
    @svg 
    @@colors = [
      "cyan","red","green","blue","orange","deeppink","lightgreen","black","purple","gray",
      "gray","cyan","purple","tomato","black","green","lightgreen","cornflowerblue","pink","orange"] * 2
    
    def self.colors() 
      return @@colors 
    end 
    @@markers   = [".","o","s","1","*","x","+","d","p","H"] * 4
    def self.markers() 
      return @@markers 
    end 
    @@linestyles = [
      "-","--","-.",":","3","10 1","10 5","5 10","5 2 2 5","3 10",
      "3 10","5 2 2 5","5 10","10 5","10 1","3",":","-.","--","-"
    ] * 2
    def self.linestyles
      return @@linestyles
    end 
  
    attr_accessor :minx,:maxx,:miny,:maxy

    attr_accessor :padleft, :padright,:padtop,:padbottom,:colors,:labels,:markers,
      :linestyles,:opacities,:linewidth


    attr_accessor :markernum,:markersize
    attr_accessor :xlabel,:ylabel,:axcolor,:axlinewidth,:axlabelstrokewidth,
      :axlabelfontsize,:xlabelformat,:ylabelformat
    attr_accessor :ticksnum,:tickslen,:tickstextlen,:ticksfontsize,:ticksstrokewidth
    attr_accessor :legendfontsize,:legendbgcolor,:legendbgopacity,:legendfgcolor,
      :legendlinelen,:legendheight,:legendpad
    attr_accessor :title, :titlefontsize
    
    def initialize(svg)
      @svg = svg

      @padleft   = 100
      @padright  = 50
      @padtop    = 50
      @padbottom = 100
      @colors    = [] 
      @labels    = []
      @markers   = [] 
      @linestyles   = [] 
      @opacities = [] 
      @linewidth = 2

      @markernum  = 10
      @markersize = 15
   
      @xlabel = ""
      @ylabel = ""
      @axcolor = "black"
      @axlinewidth = 2
      @axlabelstrokewidth = 0.5
      @axlabelfontsize = 25
      @xlabelformat = "%-0.4f"
      @ylabelformat = "%-0.4f"
   

      @ticksnum = 5
      @tickslen = 10 
      @tickstextlen = 60
      @ticksfontsize = 18
      @ticksstrokewidth = 0.1
  
      @legendfontsize = 20
      @legendbgcolor = "white"
      @legendbgopacity = 0.8
      @legendfgcolor = "black"
      @legendlinelen = 50 
      @legendheight = 100 
      @legendpad = 10 

      @title = ""
      @titlefontsize = 40 

      @minx = -1.0/0.0 
      @maxx = 1.0/0.0
      @miny = -1.0/0.0
      @maxy = 1.0/0.0

    end 

    def dark_mode
      @axcolor = "white"
      @legendbgcolor = "black"
      @legendfgcolor = "white"
    end 

    def minmax(x,y,xlog: false,ylog: false)
      y = y.flatten 
      if ylog 
        y = y
          .map{|e|begin Math.log(e,10) rescue -1.0/0.0 end } 
          .reject{|e| e.infinite?} 
      end 

      miny,maxy = y.minmax
      @miny = miny unless (@miny && @miny.finite?)
      @maxy = maxy unless (@maxy && @maxy.finite?)

      x = x.flatten 
      if xlog 
        x = x
          .map{|e|begin Math.log(e,10) rescue -1.0/0.0 end } 
          .reject{|e| e.infinite?} 
      end 
      minx,maxx = x.minmax
      @minx = minx unless (@minx && @minx.finite?)
      @maxx = maxx unless (@maxx && @maxx.finite?)
    end 



    def self.mapxcoords(x,minx,maxx,w,padleft,padright,log)
      begin 
        x = log ? Math.log(x,10) : x 
      rescue 
        STDERR.puts "WARNING error execption Math.log(#{x},10)"
        x = -1.0/0.0
      end 
      xnew = padleft + ((x - minx)/((maxx-minx) + 1e-100))*(w-(padleft + padright))
    end 

    def self.mapycoords(y,miny,maxy,h,padtop,padbottom,log)
      begin 
        y = log ? Math.log(y,10) : y 
      rescue 
        STDERR.puts "WARNING error execption Math.log(#{y},10)"
        y = -1.0/0.0
      end
      ynew = padtop + ((maxy - y)/((maxy-miny) + 1e-100))*(h-(padtop + padbottom))
    end

    def map_and_zip(xraw,yraw,minx,maxx,miny,maxy,xlog,ylog)
      xp = xraw.map{|v| Plot.mapxcoords(v,minx,maxx,@svg.width,@padleft,@padright,xlog)}
      yp = yraw.map{|v| Plot.mapycoords(v,miny,maxy,@svg.height,@padtop,@padbottom,ylog)}
      pts = xp.zip(yp)  
      pts = pts.reject{|p| p[0].infinite? || p[1].infinite?}
      return pts
    end 

    def set_plot_style(label,color,marker,linestyle,opacity)
      label = @labels.size.to_s if !label
      color = @@colors[@labels.size] if  !color 
      marker = @@markers[@labels.size] if !marker
      linestyle = @@linestyles[@labels.size] if !linestyle
      opacity = 1.0 if !opacity

      @labels.push(label)
      @colors.push(color)
      @markers.push(marker)
      @linestyles.push(linestyle)
      @opacities.push(opacity)
    end 

    def title
      @svg.g(id: "title",text_anchor: "middle") do |g| 
        g.text(
          x: @svg.width / 2, y: @titlefontsize,
          text: @title,
          font_size: @titlefontsize)
      end 
    end 

    def legend
      legendtextwidth = @legendfontsize*0.75*@labels.map{|e| e.size}.max
      legendwidth  = @legendlinelen + 3*@legendpad + legendtextwidth
      legendheight = 2*@legendpad + @labels.size*2*@legendfontsize

      t = SVG.transform do |t|
        t.translate(@svg.width-legendwidth-@padright,@padtop)
      end 

      @svg.g(id: "legend",transform: t) do |g|
        g.rect(
          x: 0, 
          y: 0 ,
          width: legendwidth, 
          height: legendheight, 
          stroke: "none" , 
          fill: @legendbgcolor, 
          opacity: @legendbgopacity)

        @labels.size.times do |i|
          y = @legendpad + @legendfontsize + (i*2*@legendfontsize)
          g.g(id: "legend-marker-line-"+@labels[i],fill: @colors[i],stroke: @colors[i]) do |g|
            g.line(
              x1: @legendpad,
              y1: y - @legendfontsize.to_i / 4, 
              x2: @legendpad+@legendlinelen, 
              y2: y - @legendfontsize.to_i / 4,
              stroke_dasharray: self.linestyle(@linestyles[i]))
            self.marker(
              @markers[i],
              @legendpad + @legendlinelen.to_i / 2,
              y - @legendfontsize.to_i / 4,
              "",
              @legendfontsize)
          end 
          g.text(x: @legendpad+@legendfontsize+@legendlinelen,y: y ,
            stroke: "none" ,
            fill: @legendfgcolor,
            text: @labels[i],
            font_size: @legendfontsize) 
        end 
      end 
    end 

    def axis(ticks,labels, flip: false, rotang: 0,**args)
      min,max = ticks[0], ticks[-1]
      # Dibujar el eje
      @svg.g(id: "axis#{flip ? @xlabel : @ylabel}", 
        text_anchor: "middle",
        stroke: @axcolor, 
        stroke_width: @axlinewidth,
        fill: @axcolor,**args) do |axis|
        axis.path() do |path|
          path.m(min[0],min[1])
          path.l(max[0],max[1])
        end 

        # Mostrar nombre
        t = SVG.transform do |t|
          if flip
            t.translate(@svg.width/2,@svg.height - @axlabelfontsize)
          else 
            t.translate(@axlabelfontsize,@svg.height/2)
            t.rotate(-90,0,0)
          end 
        end 

        axis.text(text: flip ? @xlabel : @ylabel, x: 0, y: 0, stroke_width: @axlabelstrokewidth,transform: t,
          font_size: @axlabelfontsize)

        # Colocar los ticks
        axis.path do |tickspath|
          (0 ... ticks.size).each do |i|
            x,y = ticks[i]
            tickspath.m(x-@tickslen/2,y)
            tickspath.l(@tickslen,0,relative: true)
            tickspath.m(x,y-@tickslen/2)
            tickspath.l(0,@tickslen,relative: true)
          end 
        end 

        # Colocar los valores 
        (0 ... ticks.size).each do |i| 
          x,y = ticks[i]
          x +=  flip ? 0 : -@tickslen 
          y += !flip ? 0 : @tickslen + @ticksfontsize*1.5
          text = labels[i]
          axis.text(
            x: x, 
            y: y , 
            text: text, 
            text_anchor: flip ? "middle" : "end",
            stroke_width: @ticksstrokewidth,
            font_size: @ticksfontsize,
            transform: SVG.transform{|t| t.rotate(rotang,x,y)})
        end 
      end 
    end

    def yticks(min: @miny,max: @maxy,frmt: @ylabelformat, ticksnum: @ticksnum)
      ticks = []
      labels = []

      ticks.push([@padleft,Plot.mapycoords(min,min,max,@svg.height,@padtop,@padbottom,false)])
      labels.push(frmt % min)
        ticksnum.times do |i|
          v = min + ((max - min) / (ticksnum+1)) * (i+1)
          ticks.push([@padleft,Plot.mapycoords(v,min,max,@svg.height,@padtop,@padbottom,false)])
          labels.push(frmt % v)
        end 
        ticks.push([@padleft,Plot.mapycoords(max,min,max,@svg.height,@padtop,@padbottom,false)])
        labels.push(frmt % max)
        return ticks,labels
    end 

    def boxplotticks(nboxes,boxscale = 0.5)
      ticks = []
      labels = []

      w = (@svg.width - (@padleft + @padright)) / nboxes
      boxwidth = boxscale * w

      ticks.push([@padleft,@svg.height - @padbottom])
      labels.push("")
      nboxes.times do |i| 
        x =  @padleft + w*(i.to_f+0.5)
        ticks.push([x,@svg.height - @padbottom])
        labels.push("%d" % i)
      end 

      ticks.push([@svg.width - @padright,@svg.height - @padbottom])
      labels.push("")
      return ticks,labels
    end 

    def xticks(min: @minx, max: @maxx, ticksnum: @ticksnum, frmt: @xlabelformat)
      ticks = []
      labels = []

      ticks.push([Plot.mapxcoords(min,min,max,@svg.height,@padleft,@padright,false),@svg.height-@padbottom])
      labels.push(frmt % min)
      ticksnum.times do |i|
        v = min + ((max - min) / (ticksnum+1)) * (i+1)
        ticks.push([Plot.mapxcoords(v,min,max,@svg.height,@padleft,@padright,false),@svg.height-@padbottom])
        labels.push(frmt % v)
      end 
      ticks.push([Plot.mapxcoords(max,min,max,@svg.height,@padleft,@padright,false),@svg.height-@padbottom])
      labels.push(frmt % max)
      return ticks,labels

    end 

    def linestyle(style)
      case style
      when "-" then "5 0"
      when "--","dashed" then "5"
      when "-.","dashdot" then "6 3 3 3"
      when ":","dotted"  then "2"
      when " ","","none" then "0 1"
      else style
      end 
    end 


  end 
end 
require "plot2d/markers.rb"
require "plot2d/scatter.rb"
require "plot2d/lineplot.rb"
require "plot2d/boxplot.rb"
