class SVG
class Plot

  def boxplot(
      boxesdata,
      colors: @@colors,
      markers: @@markers,
      linewidths: nil,
      labels: nil,
      opacity: nil,
      ylog: false, **args)

    @colors = colors.dup
    @markers = markers.dup 
    @labels = labels.dup 
    linewidths = linewidths || @colors.size 

    #
    median = lambda do |arr|
      len = arr.size
      if arr.size.even? 
        return (arr[len/2] + arr[(len+1)/2])/2.0
      else 
        return arr[len/2]
      end
    end 

    mean = lambda do |arr|
      return arr.reduce(0.0){|acc,d| acc + d} / arr.size 
    end 

    std = lambda do |arr,mean|
      return (arr.reduce(0.0){|acc,d| acc + (d - mean)**2}/(arr.size-1))**0.5
    end 
 

    # Cantidad de cajas
    nboxes = boxesdata.size 
    min,max = boxesdata.flatten.minmax

    # Obtener los estadísticos necesarios para el boxplot
    boxes = [ ]
    nboxes.times do |i|
      box = {}
      data = boxesdata[i] 
    
      sz = data.size; 
      data = data.sort
      box[:mean] = mean.call(data)
      box[:q1] = median.call(data[0..data.size/2])
      box[:q2] = median.call(data) 
      box[:q3] = median.call(data[(data.size/2)+1..-1])
      box[:iqr] = box[:q3] - box[:q1]
      box[:median] = box[:q2]
      box[:std] = std.call(data,box[:mean])
    
      box[:w1] = data[0]
      (0 ... data.size).each do |i|
        if data[i]  >= (box[:q1] - (1.5*box[:iqr]))
          box[:w1] = data[i]
          break
        end 
      end 
      box[:w2] = data[-1]
      (0 ... data.size).reverse_each do |i|
        if data[i]  <= (box[:q3] + (1.5*box[:iqr]))
          box[:w2] = data[i]
          break
        end 
      end 

      boxes.push(box)
    end 

    # Modificar el máximo y mínimo para mostrar mejor los boxplots 
    @maxy = boxes.map{|b| b[:w2]*1.01}.max 
    # @miny = boxes.map{|b| b[:w1]*0.1}.max

    boxscale = 0.5
    w = (@svg.width - (@padleft + @padright))  / nboxes
    boxes.each_with_index do |box,i|
      @svg.g(id: @labels.last,
        stroke: @colors[i], 
        fill: @colors[i],
        stroke_width: @linewidth,
        opacity: @opacities.last) do |g|

        boxwidth = boxscale * w
        x = @padleft + w*(i.to_f+0.5)

        ymedian = Plot.mapycoords(box[:median],@miny,@maxy,@svg.height,@padtop,@padbottom,ylog)

        # median line
        g.line(x1: x - boxwidth/2, y1: ymedian, x2: x + boxwidth/2, y2: ymedian, stroke: "black")

        # mean line 
        ymean = Plot.mapycoords(box[:mean],@miny,@maxy,@svg.height,@padtop,@padbottom,ylog)
        g.line(
          x1: x - boxwidth/2, y1: ymean, 
          x2: x + boxwidth/2, y2: ymean, 
          stroke_dasharray: self.linestyle("--"),
          stroke: "black")

        # box 
        yq3 = Plot.mapycoords(box[:q3],@miny,@maxy,@svg.height,@padtop,@padbottom,ylog)
        yq1 = Plot.mapycoords(box[:q1],@miny,@maxy,@svg.height,@padtop,@padbottom,ylog)

        g.rect(x: x-(boxwidth/2), y: yq3, width: boxwidth, height: (yq3-yq1).abs,opacity: 0.2)
        g.rect(x: x-(boxwidth/2), y: yq3, width: boxwidth, height: (yq3-yq1).abs,fill: "none", stroke: "black")
       
        # whiskers
        wisker_width = boxwidth/2
        w1 = Plot.mapycoords(box[:w1],@miny,@maxy,@svg.height,@padtop,@padbottom,ylog)
        w2 = Plot.mapycoords(box[:w2],@miny,@maxy,@svg.height,@padtop,@padbottom,ylog)
       
        g.path(stroke_dasharray: self.linestyle("-"),stroke: "black") do |wp|
          wp.m(x,w2)
          wp.m(-wisker_width/2,0,relative: true)
          wp.l(wisker_width,0,relative: true)
          wp.m(x,w2)
          wp.l(x,yq3)

          wp.m(x,w1)
          wp.m(-wisker_width/2,0,relative: true)
          wp.l(wisker_width,0,relative: true)
          wp.m(x,w1)
          wp.l(x,yq1)
        end  
        # Draw outliers
        boxesdata[i].each do |y| 
          next if  (y >= box[:w1] && y <= box[:w2])
          y = Plot.mapycoords(y,@miny,@maxy,@svg.height,@padtop,@padbottom,ylog)
          self.marker(@markers[i],x,y,"%g" % y,8)
        end  
       
      end 
    end 
    
  end 

end 
end 
