class SVG
  class Plot 
    def marker(marker,x,y,val,ms = @markersize)
      t = SVG.transform do |t|
        t.translate(x,y)
        t.scale(ms)
        case marker 
        when "D","x","X" then t.rotate(45)
        when ">","2","H" then t.rotate(90)
        when "v","3" then t.rotate(180)
        when "<","4" then t.rotate(270)
        else nil
        end 
      end 

      case marker 
      when "." then self.marker_point(t,val)
      when "," then self.marker_pixel(t,val)
      when "o" then self.marker_circle(t,val)
      when "^",">","v","<" then self.marker_triangle(t,val)
      when "s","D" then self.marker_square(t,val)
      when "d" then self.marker_diamond(t,val)
      when "+","x" then self.marker_plus(t,0.1,val)
      when "P","X" then self.marker_plus(t,0.2,val)
      when "1","2","3","4" then self.marker_tri(t,val)
      when "h","H" then self.marker_hexagon(t,val)
      when "p" then self.marker_pentagon(t,val)
      when "8" then self.marker_octagon(t,val)
      when "*" then self.marker_star(t,val)
      else raise "unkown marker #{marker}"
      end 
    end 
    
    def marker_point(t,val="")
      @svg.circle(cx: 0, cy: 0, r: 0.3, transform: t,title: val,stroke_width: 0)
    end 

    def marker_pixel(t,val="")
      @svg.circle(cx: 0, cy: 0, r: 0.2, transform: t,title: val,stroke_width: 0)
    end 

    def marker_circle(t,val="")
      @svg.circle(cx: 0, cy: 0, r: 0.45, transform: t,title: val,stroke_width: 0)
    end 
    
    def marker_triangle(t,val="")
      @svg.path(transform: t, title: val,stroke_width: 0) do |path|
        path.m(0,-0.5)
        path.l(0.5,0.5)
        path.l(-0.5,0.5)
        path.z
      end 
    end 

    def marker_square(t,val="")
      @svg.path(transform: t, title: val,stroke_width: 0) do |path|
        path.m(-0.4,-0.4)
        path.h(0.4)
        path.v(0.4)
        path.h(-0.4)
        path.z
      end 
    end 

    def marker_diamond(t,val="")
      @svg.path(transform: t, title: val,stroke_width: 0) do |path|
        path.m(0,-0.5)
        path.l(0.3,0)
        path.l(0,0.5)
        path.l(-0.3,0.0)
        path.z
      end 
    end 

    def marker_plus(t,width,val="")
      @svg.path(transform: t, title: val,stroke_width: 0 ) do |path|
        path.m(-width,-0.5)
        path.h(width)
        path.v(0.5)
        path.h(-width)
        path.z
      end 
      @svg.path(transform: t, title: val,stroke_width: 0 ) do |path|
        path.m(-0.5,-width)
        path.v(width)
        path.h(0.5)
        path.v(-width)
        path.z
      end 
    end 

    def marker_tri(t,val="")
      @svg.g(title: val, transform: t,stroke_width: 0) do |g|
        width = 0.1
        @svg.path(title: val) do |path|
          path.m(-width,0.5)
          path.h(width)
          path.v(0.0)
          path.h(-width)
          path.z
        end 
        @svg.path(
          title: val,
          transform: SVG.transform{|t| t.rotate(135)}) do |path|
          path.m(-width,0.5)
          path.h(width)
          path.v(0.0)
          path.h(-width)
          path.z
        end 
        @svg.path(
          title: val,
          transform: SVG.transform{|t| t.rotate(225)}) do |path|
          path.m(-width,0.5)
          path.h(width)
          path.v(0.0)
          path.h(-width)
          path.z
        end 
      end 
    end 
    
    def marker_hexagon(t,val="")
      @svg.path(title: val, transform: t,stroke_width: 0) do |path|
        path.m( 0.00,-0.50)
        path.l( 0.40,-0.20)
        path.l( 0.40, 0.30)
        path.l( 0.00, 0.50)
        path.l(-0.40, 0.30)
        path.l(-0.40,-0.20)
        path.z
      end 
    end 

    def marker_pentagon(t,val="")
      @svg.path(title: val, transform: t,stroke_width: 0) do |path|
        path.m( 0.00,-0.50)
        path.l( 0.45,-0.10)
        path.l( 0.30, 0.50)
        path.l(-0.30, 0.50)
        path.l(-0.45,-0.10)
        path.z
      end 
    end 

    def marker_octagon(t,val="")
      @svg.path(title: val, transform: t,stroke_width: 0) do |path|
        path.m(-0.25,-0.50)
        path.l( 0.18,-0.50)
        path.l( 0.50,-0.25)
        path.l( 0.50, 0.18)
        path.l( 0.18, 0.50)
        path.l(-0.25, 0.50)
        path.l(-0.50, 0.18)
        path.l(-0.50,-0.25)
        path.z
      end 
    end 

    def marker_star(t,val="")
      @svg.path(title: val, transform: t,stroke_width: 0) do |path|
        path.m(0,-0.5)
        path.l(-0.3,0.5)
        path.l(0.5,-0.1)
        path.l(-0.5,-0.1)
        path.l(0.3,0.5)
        path.z
      end 
    end 
  end 
end 
